#include <QWidget>
#include <QLabel>
#include <QBoxLayout>
#include <QApplication>
#include <QScriptEngine>
#include <QSpinBox>
#include <QMessageBox>
#include <QCheckBox>
#include <QPushButton>

class DataConnector : public QObject
{
    Q_OBJECT
public:
    DataConnector(QObject *parent) :
        QObject(parent),
        m_searchValue(-1),
        m_currentValue(-1),
        m_engine(),
        m_scriptSource("f()"),
        m_scriptProgram(m_scriptSource),
        m_useProgram(false)
    {
        QScriptValue dataConnector = m_engine.newQObject(this, QScriptEngine::QtOwnership, QScriptEngine::ExcludeChildObjects );
        m_engine.globalObject().setProperty("dataConnector", dataConnector);
        m_engine.evaluate(
            "function f() {\n"
            "	var x = dataConnector.currentValue();\n"
            "	if( x < 0 )\n"
            "		x = dataConnector.increment();\n"
            "	return x;\n"
            "}\n"
        );
    }

    int	calc(int x)
    {
        m_currentValue=0;
        m_searchValue=x;
        QScriptValue v = m_useProgram ? m_engine.evaluate(m_scriptProgram) : m_engine.evaluate(m_scriptSource);
        if( m_engine.hasUncaughtException() )
        {
            QMessageBox::information(0,QString("Error"),m_engine.uncaughtException().toString(), QMessageBox::Ok, QMessageBox::NoButton);
        }
        return v.toInt32();
    }

public slots:
    void setUsePrecompiledProgram( bool f )
    {
        m_useProgram = f;
    }

    virtual int currentValue() const
    {
        return m_currentValue == m_searchValue ? m_searchValue : -1;
    }

    virtual int increment()
    {
        ++m_currentValue;
        QMessageBox::information(0,"Hint",QString::number(m_currentValue));
        QScriptValue result = m_useProgram ? m_engine.evaluate(m_scriptProgram) : m_engine.evaluate(m_scriptSource);
        return result.toInt32();
    }

private:
    int					m_searchValue;
    int					m_currentValue;
    QScriptEngine		m_engine;
    QString				m_scriptSource;
    QScriptProgram		m_scriptProgram;
    bool				m_useProgram;
};

class Widget : public QWidget
{
    Q_OBJECT
public:
    Widget( QWidget* parent=0 ) :
        QWidget(parent),
        m_useProgramCheck(new QCheckBox("Use program",this)),
        m_spinBox(new QSpinBox(this)),
        m_calcButton(new QPushButton("Calc",this)),
        m_resultBox(new QSpinBox(this)),
        m_dataConnector(new DataConnector(this))
    {
        new QBoxLayout(QBoxLayout::LeftToRight,this);
        layout()->addWidget(m_useProgramCheck);
        layout()->addWidget(m_spinBox);
        layout()->addWidget(m_calcButton);
        layout()->addWidget(m_resultBox);
        connect(m_calcButton,SIGNAL(clicked()),this,SLOT(calc()));
        connect(m_useProgramCheck, SIGNAL(toggled(bool)),m_dataConnector,SLOT(setUsePrecompiledProgram(bool)));
    }
protected slots:
    void calc()
    {
        int result = m_dataConnector->calc(m_spinBox->value());
        m_resultBox->setValue(result);
    }

private:
    QCheckBox*			m_useProgramCheck;
    QSpinBox*			m_spinBox;
    QPushButton*		m_calcButton;
    QSpinBox*			m_resultBox;
    DataConnector*		m_dataConnector;
};

#include "main.moc"

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    Widget w;
    w.show();
    return a.exec();
}
